/**
 * 工具函数
 */

const path = require('path')
const fs = require('fs')
const chalk = require('chalk')
const { version } = require('../../package.json')

process.env.mode = 'production'
process.env.projectRoot = process.cwd()
process.env.customConfig = ''

/**
 * 设置缓存 mode
 */
const setMode = mode => (process.env.mode = mode)
exports.setMode = setMode

/**
 * 获取缓存 mode
 */
const getMode = () => process.env.mode
exports.getMode = getMode

/**
 * 获取项目根目录
 */
const getProjectRoot = () => process.env.projectRoot
exports.getProjectRoot = getProjectRoot

/**
 * 设置自定义配置
 */
const setCustomConfig = config => {
  process.env.customConfig = JSON.stringify(config)
}
exports.setCustomConfig = setCustomConfig

/**
 * 获取自定义配置
 */
const getCustomConfig = () => {
  try {
    return JSON.parse(process.env.customConfig)
  } catch (error) {
    return {}
  }
}
exports.getCustomConfig = getCustomConfig

/**
 * 扩展配置
 */
exports.extendConfig = () => {
  const extendConfigScript = path.resolve(getProjectRoot(), 'legoflow.js')
  const extendConfigFunc = fs.existsSync(extendConfigScript) ? require(extendConfigScript) : void 0

  switch (typeof extendConfigFunc) {
    case 'function':
      return extendConfigFunc
    case 'object':
      extendConfigFunc && setCustomConfig(extendConfigFunc)
      break
  }
}

/**
 * 获取 sourceMap
 */
exports.getSourceMap = () => {
  const mode = getMode()
  const customConfig = getCustomConfig()

  if (mode === 'production' && customConfig['build/sourceMap'] === true) {
    return 'source-map'
  }

  return mode === 'development' ? 'inline-source-map' : ''
}

/**
 * 获取是否带 hash
 */
exports.isWithHash = () => {
  const mode = getMode()
  const customConfig = getCustomConfig()

  if (mode === 'production' && customConfig['build/hash'] === false) {
    return ''
  }

  return mode === 'development' ? '' : '.[hash]'
}

/**
 * 打印版本号
 */
exports.logVersion = () => console.log(chalk.blue.bold(`LegoFlow Engine v${version}`))

/**
 * 获取项目信息
 */
exports.getProjectInfo = () => {
  const projectPackageJson = path.resolve(getProjectRoot(), 'package.json')

  if (!fs.existsSync(projectPackageJson)) {
    return {}
  }

  try {
    return JSON.parse(fs.readFileSync(projectPackageJson, 'utf8'))
  } catch (error) {
    return {}
  }
}
